# -*- coding: ISO-8859-1 -*-
# Copyright (C) 2003-2005 Juan David Ibez Palomar <jdavid@itaapy.com>
#               2004 Jean-Philippe Robles <jpr@itaapy.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# Import from the Standard Library
from time import time

# Import from itools
from itools import get_abspath
from itools.resources import get_resource, memory
from itools.xml import XML
from itools.xhtml import XHTML
from itools import i18n
from itools.zope import get_context

# Import from ikaaro
from Folder import Folder
from utils import reduce_string



class Skin(Folder):

    class_id = 'Skin'
    class_title = u'Skin'
    class_icon16 = 'images/Skin16.png'
    class_icon48 = 'images/Skin48.png'

    __fixed_handlers__ = ['template.xhtml.en']


    def get_breadcrumb(self):
        """Return a list of dicts [{name, url}...] """
        context = get_context()
        here = context.handler

        #
        handlers = []
        handler = here
        while handler is not None:
            handlers.insert(0, handler)
            handler = handler.parent

        # 
        breadcrumb = []
        for handler in handlers:
            # The link
            view = handler.get_firstview()
            if view is None:
                url = here.get_pathto(handler)
            else:
                url = '%s/;%s' % (here.get_pathto(handler), view)
            # The title
            title = handler.get_title_or_name()
            short_title = reduce_string(title, 15, 30) 
            # Name
            breadcrumb.append({'name': title, 'short_name': short_title,
                               'url': url})

        return breadcrumb


    def get_metadata(self):
        context = get_context()
        request = context.request
        handler = context.handler

        namespace = {}

        namespace['title'] = handler.get_property('dc:title')
        namespace['format'] = handler.class_title
##        namespace['language'] = language
        namespace['mtime'] = handler.get_mtime().strftime('%Y-%m-%d %H:%M')

        # Languages
        site_root = handler.get_site_root()
        site_languages = site_root.get_property('ikaaro:website_languages')
        language = request.get_cookie('content_language') or site_languages[0]
        languages = []
        for x in site_languages:
            languages.append({'code': x,
                              'name': i18n.get_language_name(x),
                              'is_selected': x == language})
        namespace['languages'] = languages

        return namespace


    def get_tabs(self):
        """
        Return tabs and subtabs as a dict {tabs, subtabs} of list of dicts
        [{name, label, active, style}...].
        """
        # Get request, path, etc...
        context = get_context()
        handler = context.handler

        # Tabs
        views = handler.get_views()
        subviews = handler.get_subviews(context.method)

        tabs = []
        for name in views:
            method = handler.get_method(name)
            if method is not None:
                label = getattr(method.im_class, '%s__label__' % name)
                active = name == context.method or name in subviews
                tabs.append({'name': ';%s' % name,
                             'label': handler.gettext(label),
                             'active': active,
                             'style': active and 'tab_active' or 'tab'})

        # Subtabs
        subtabs = []
        for name in subviews:
            method = handler.get_method(name)
            if method is not None:
                label = getattr(method.im_class, '%s__sublabel__' % name)
                active = name == context.method
                subtabs.append({'name': ';%s' % name,
                                'label': handler.gettext(label),
                                'active': active,
                                'style':  active and 'tab active' or 'tab'})

        return {'tabs':tabs, 'subtabs':subtabs}


##    def get_languages(self):
##        """
##        Compute available languages.
##        Return of list of dicts [{id, title, selected}...]
##        """
##        context = get_context()
##        request = context.request
##        root, handler = context.root, context.handler

##        language_action = '%s/;change_language' % handler.get_pathto(root)

##        accept = request.accept_language
##        available_languages = root.get_available_languages()
##        default_language = root.get_default_language()
##        selected_language = accept.select_language(available_languages) \
##                            or default_language

##        languages = [ {'id': x, 'title': i18n.get_language_name(x),
##                       'selected': x == selected_language}
##                      for x in available_languages ]

##        return {'language_action': language_action, 'languages': languages}


    def get_user_menu(self):
        """Return a dict {user_icon, user, joinisopen}."""
        context = get_context()
        user = context.user
        here = context.handler
        root = context.root

        path_to_root = here.get_pathto(root)
        if user is None:
            joinisopen = root.get_property('ikaaro:website_is_open')
        else:
            joinisopen = None
            home = '%s/;%s' % (here.get_pathto(user), user.get_firstview())
            user = {'name': user.name,
                    'title': user.title or user.name,
                    'home': home,
                    'logout': '%s/;logout' % path_to_root}
        return {'user_icon': '%s/user16' % path_to_root,
                'user': user,
                'joinisopen': joinisopen,
                'login_url': '%s/;login_form' % path_to_root,
                'join_url': '%s/;join_form' % path_to_root}


    def get_message(self):
        """Return a message string from de request."""
        request = get_context().request
        message = request.form.get('message', None)
        return message


    def build_namespace(self):
        context = get_context()

        namespace = {}

        # Resources
        namespace['styles'] = [ x for x in context.styles ]
        namespace['scripts'] = [ x for x in context.scripts ]

        # User menu
        namespace['user_icon'] = self.get_user_menu()['user_icon']
        namespace['user'] = self.get_user_menu()['user']
        namespace['joinisopen'] = self.get_user_menu()['joinisopen']
        namespace['login_url'] = self.get_user_menu()['login_url']
        namespace['join_url'] = self.get_user_menu()['join_url']

        # Languages
##        languages = self.get_languages()
##        namespace['language_action'] = languages['language_action']
##        namespace['languages'] = languages['languages']

        # Breadcrumb
        namespace['breadcrumb'] = self.get_breadcrumb()

        # Metadata
        namespace['metadata'] = self.get_metadata()

        # Tabs
        tabs = self.get_tabs()
        namespace['tabs'] = tabs['tabs']
        namespace['subtabs'] = tabs['subtabs']

        # Message
        namespace['message'] = self.get_message()

        return namespace


    def set_template_prefix(self, handler):
        context = get_context()

        # Set the prefix
        here = context.handler
        data = []
        data.append(handler.header_to_unicode())
        # Let's go!
        for node, context in handler.traverse2():
            if isinstance(node, XML.Element):
                if context.start:
                    data.append(u'<%s' % node.qname)
                    for namespace, local_name, value in node.get_attributes():
                        qname = node.get_attribute_qname(namespace, local_name)
                        if local_name in ('href', 'src') \
                           and not value.scheme and not value.authority:
                            if value.path.is_relative():
                                # XXX Here we loss the query and fragment.
                                x = self.get_handler(value.path)
                                value = here.get_pathto(x)
                            else:
                                value = str(value)[1:]
                                value = here.get_pathtoroot() + value
                        else:
                            type = node.get_attribute_type(namespace,
                                                           local_name)
                            value = type.encode(value)
                        data.append(u' %s="%s"' % (qname, value))
                    data.append(u'>')
                else:
                    data.append(u'</%s>' % node.qname)
            elif isinstance(node, XML.Comment):
                data.append(node.to_unicode())
            elif isinstance(node, unicode):
                data.append(node)
            else:
                raise ValueError, 'unexpected value "%s"' % node

        data = u''.join(data)
        data = data.encode('utf8')
        resource = memory.File(data)
        handler = XHTML.Document(resource)

        return handler


    def template(self, content):
        context = get_context()
        # Build the namespace
        namespace = self.build_namespace()

        # Content
        namespace['body'] = content
        namespace['handler'] = context.handler

        # Set the encoding to UTF-8
        context.response.set_header('Content-Type', 'text/html; charset=UTF-8')

        # Transform the tree
        handler = self.get_handler('template.xhtml')
        handler = self.set_template_prefix(handler)

        # STL
        s = []
        header = handler.header_to_unicode()
        # XXX Strip XML declaration, because it makes IE6 fall into quirks
        # mode (see http://hsivonen.iki.fi/doctype/)
        header = header.split('\n', 1)[1]
        s.append(header)
        s.append(handler.stl(namespace))
        return u''.join(s)


Folder.register_handler_class(Skin)


#############################################################################
# The folder "/ui"
#############################################################################

skin_registry = {}
def register_skin(name, skin):
    if isinstance(skin, str):
        resource = get_resource(skin)
        skin = Skin(resource)
    skin_registry[name] = skin


# Register the built-in skins
path = get_abspath(globals(), 'ui')
register_skin('classic', '%s/classic' % path)
register_skin('surf', '%s/surf' % path)


class UI(Folder):

    def _get_handler(self, segment, resource):
        name = segment.name
        if name in skin_registry:
            return skin_registry[name]
        return Folder._get_handler(self, segment, resource)


    def _get_virtual_handler(self, segment):
        name = segment.name
        if name in skin_registry:
            return skin_registry[name]
        return Folder._get_virtual_handler(self, segment)


path = get_abspath(globals(), 'ui')
resource = get_resource(path)
ui = UI(resource)
